{% extends "admin/change_form.html" %}
{% load i18n %}

{% block object-tools-items %}
    {{ block.super }}
    {% with original.get_stripe_dashboard_url as stripe_url %}
    {% if stripe_url %}
        <li><a href="{{ stripe_url }}" target="_blank" id="djstripe-view-on-stripe"><i class="icon-user icon-alpha75"></i>{% trans "View on Stripe Dashboard" %}</a></li>
    {% endif %}
    {% endwith %}
{% endblock %}
